@@ -22,11 +22,6 @@
 #include "config.h"
 #endif
 
-#if defined(__MINGW32__)
-/* ftello64() and fseeko64() prototypes from stdio.h */
-#   undef __STRICT_ANSI__
-#endif
-
 #include "file.h"
 #include "util/macro.h"
 #include "util/logging.h"
@@ -35,10 +30,6 @@
 #include <stdlib.h>
 #include <inttypes.h>
 
-#ifdef _WIN32
-#include <windows.h>
-#endif	//	#ifdef _WIN32
-
 static void file_close_linux(BD_FILE_H *file)
 {
     if (file) {
@@ -52,24 +43,12 @@ static void file_close_linux(BD_FILE_H *file)
 
 static int64_t file_seek_linux(BD_FILE_H *file, int64_t offset, int32_t origin)
 {
-#if defined(__MINGW32__)
-    return fseeko64((FILE *)file->internal, offset, origin);
-#elif defined(_WIN32)
-    return _fseeki64((FILE *)file->internal, offset, origin);
-#else
     return fseeko((FILE *)file->internal, offset, origin);
-#endif
 }
 
 static int64_t file_tell_linux(BD_FILE_H *file)
 {
-#if defined(__MINGW32__)
-    return ftello64((FILE *)file->internal);
-#elif defined(_WIN32)
-    return _ftelli64((FILE *)file->internal);
-#else
     return ftello((FILE *)file->internal);
-#endif
 }
 
 static int file_eof_linux(BD_FILE_H *file)
@@ -77,8 +56,6 @@ static int file_eof_linux(BD_FILE_H *file)
     return feof((FILE *)file->internal);
 }
 
-#define BD_MAX_SSIZE ((int64_t)(((size_t)-1)>>1))
-
 static int64_t file_read_linux(BD_FILE_H *file, uint8_t *buf, int64_t size)
 {
     if (size > 0 && size < BD_MAX_SSIZE) {
@@ -112,14 +89,7 @@ static BD_FILE_H *file_open_linux(const char* filename, const char *mode)
     file->tell = file_tell_linux;
     file->eof = file_eof_linux;
 
-#ifdef _WIN32
-    wchar_t wfilename[MAX_PATH], wmode[8];
-    if (MultiByteToWideChar(CP_UTF8, MB_ERR_INVALID_CHARS, filename, -1, wfilename, MAX_PATH) &&
-        MultiByteToWideChar(CP_UTF8, MB_ERR_INVALID_CHARS, mode, -1, wmode, 8) &&
-        (fp = _wfopen(wfilename, wmode))) {
-#else
     if ((fp = fopen(filename, mode))) {
-#endif
         file->internal = fp;
 
         return file;
@@ -138,21 +108,3 @@ BD_FILE_OPEN file_open_default(void)
 {
     return file_open_linux;
 }
-
-/*
- *
- */
-
-int64_t file_size(BD_FILE_H *fp)
-{
-    int64_t pos    = file_tell(fp);
-    int64_t res1   = file_seek(fp, 0, SEEK_END);
-    int64_t length = file_tell(fp);
-    int64_t res2   = file_seek(fp, pos, SEEK_SET);
-
-    if (res1 < 0 || res2 < 0 || pos < 0 || length < 0) {
-        return -1;
-    }
-
-    return length;
-}
