@@ -48,11 +48,12 @@
 #endif
 
 typedef jint (JNICALL * fptr_JNI_CreateJavaVM) (JavaVM **pvm, void **penv,void *args);
+typedef jint (JNICALL * fptr_JNI_GetCreatedJavaVMs) (JavaVM **vmBuf, jsize bufLen, jsize *nVMs);
 
 #if defined(_WIN32) && !defined(HAVE_BDJ_J2ME)
 static void *_load_jvm_win32(const char **p_java_home)
 {
-    static const char *java_home = NULL;
+    static char java_home[256] = "";
 
     wchar_t buf_loc[4096] = L"SOFTWARE\\JavaSoft\\Java Runtime Environment\\";
     wchar_t buf_vers[128];
@@ -94,9 +95,9 @@ static void *_load_jvm_win32(const char **p_java_home)
 
     if (r == ERROR_SUCCESS) {
         /* do not fail even if not found */
-        WideCharToMultiByte(CP_UTF8, 0, buf_loc, -1, strbuf, sizeof(strbuf), NULL, NULL);
-        *p_java_home = java_home = str_dup(strbuf);
-        BD_DEBUG(DBG_BDJ, "JavaHome: %s\n", strbuf);
+        WideCharToMultiByte(CP_UTF8, 0, buf_loc, -1, java_home, sizeof(java_home), NULL, NULL);
+        *p_java_home = java_home;
+        BD_DEBUG(DBG_BDJ, "JavaHome: %s\n", java_home);
     }
 
     dSize = sizeof(buf_loc);
@@ -125,12 +126,12 @@ static void *_jvm_dlopen(const char *java_home, const char *jvm_dir, const char
 {
     if (java_home) {
         char *path = str_printf("%s/%s/%s", java_home, jvm_dir, jvm_lib);
-        BD_DEBUG(DBG_BDJ|DBG_CRIT, "Opening %s ...\n", path);
+        BD_DEBUG(DBG_BDJ, "Opening %s ...\n", path);
         void *h = dl_dlopen(path, NULL);
         X_FREE(path);
         return h;
     } else {
-        BD_DEBUG(DBG_BDJ|DBG_CRIT, "Opening %s ...\n", jvm_lib);
+        BD_DEBUG(DBG_BDJ, "Opening %s ...\n", jvm_lib);
         return dl_dlopen(jvm_lib, NULL);
     }
 }
@@ -153,7 +154,11 @@ static void *_load_jvm(const char **p_java_home)
     static const char  jvm_dir[]  = "jre/bin/server";
     static const char  jvm_lib[]  = "jvm";
 # else
-    static const char *jvm_path[] = {NULL, JDK_HOME, "/usr/lib/jvm/default-java/"};
+    static const char *jvm_path[] = {NULL, JDK_HOME,
+				     "/usr/lib/jvm/default-java",
+				     "/usr/lib/jvm/java-6-openjdk",
+				     "/usr/lib/jvm/java-7-openjdk",
+				     "/etc/java-config-2/current-system-vm"};
     static const char  jvm_dir[]  = "jre/lib/" JAVA_ARCH "/server";
     static const char  jvm_lib[]  = "libjvm";
 # endif
@@ -176,7 +181,7 @@ static void *_load_jvm(const char **p_java_home)
     }
 #endif
 
-    BD_DEBUG(DBG_BDJ | DBG_CRIT, "JAVA_HOME not set, trying default locations\n");
+    BD_DEBUG(DBG_BDJ, "JAVA_HOME not set, trying default locations\n");
 
     /* try our pre-defined locations */
     for (path_ind = 0; !handle && path_ind < sizeof(jvm_path) / sizeof(jvm_path[0]); path_ind++) {
@@ -198,8 +203,7 @@ static const char *_find_libbluray_jar(void)
 #ifdef _WIN32
         "" BDJ_JARFILE,
 #else
-        "/usr/lib/libbluray/" BDJ_JARFILE,
-        "/usr/share/libbluray/" BDJ_JARFILE,
+        "/usr/share/java/" BDJ_JARFILE,
 #endif
     };
 
@@ -308,7 +312,7 @@ static int _bdj_init(BDJAVA *bdjava, JNIEnv *env)
     jclass init_class;
     jmethodID init_id;
     if (!bdj_get_method(env, &init_class, &init_id,
-                        "org/videolan/Libbluray", "init", "(JLjava/lang/String;)V")) {
+                        "org/videolan/Libbluray", "init", "(JLjava/lang/String;Ljava/lang/String;)V")) {
         return 0;
     }
 
@@ -317,8 +321,9 @@ static int _bdj_init(BDJAVA *bdjava, JNIEnv *env)
     jlong param_bdjava_ptr = (jlong)(intptr_t) bdjava;
     jstring param_disc_id = (*env)->NewStringUTF(env,
                                                  id ? id->disc_id : "00000000000000000000000000000000");
+    jstring param_disc_root = (*env)->NewStringUTF(env, bdjava->path);
     (*env)->CallStaticVoidMethod(env, init_class, init_id,
-                                 param_bdjava_ptr, param_disc_id);
+                                 param_bdjava_ptr, param_disc_id, param_disc_root);
 
     if ((*env)->ExceptionOccurred(env)) {
         (*env)->ExceptionDescribe(env);
@@ -327,8 +332,9 @@ static int _bdj_init(BDJAVA *bdjava, JNIEnv *env)
 
     (*env)->DeleteLocalRef(env, init_class);
     (*env)->DeleteLocalRef(env, param_disc_id);
+    (*env)->DeleteLocalRef(env, param_disc_root);
 
-    free(id_path);
+    X_FREE(id_path);
     bdid_free(&id);
 
     return 1;
@@ -356,41 +362,40 @@ int bdj_jvm_available(void)
     return 2;
 }
 
-BDJAVA* bdj_open(const char *path, struct bluray *bd,
-                 struct indx_root_s *index,
-                 bdj_overlay_cb osd_cb, struct bd_argb_buffer_s *buf)
+static int _find_jvm(void *jvm_lib, JNIEnv **env, JavaVM **jvm)
 {
-    BD_DEBUG(DBG_BDJ, "bdj_open()\n");
-
-    // first load the jvm using dlopen
-    const char *java_home = NULL;
-    void* jvm_lib = _load_jvm(&java_home);
-
-    if (!jvm_lib) {
-        BD_DEBUG(DBG_BDJ | DBG_CRIT, "Wasn't able to load JVM\n");
-        return NULL;
+    fptr_JNI_GetCreatedJavaVMs JNI_GetCreatedJavaVMs_fp = (fptr_JNI_GetCreatedJavaVMs)(intptr_t)dl_dlsym(jvm_lib, "JNI_GetCreatedJavaVMs");
+    if (JNI_GetCreatedJavaVMs_fp == NULL) {
+        BD_DEBUG(DBG_BDJ | DBG_CRIT, "Couldn't find symbol JNI_GetCreatedJavaVMs.\n");
+        return 0;
     }
 
+    jsize nVMs = 0;
+    JavaVM* javavm = NULL;
+
+    int result = JNI_GetCreatedJavaVMs_fp(&javavm, 1, &nVMs);
+    if (result == JNI_OK && nVMs > 0) {
+      *jvm = javavm;
+      (**jvm)->AttachCurrentThread(*jvm, (void**)env, NULL);
+      return 1;
+    }
+
+    return 0;
+}
+
+static int _create_jvm(void *jvm_lib, const char *java_home, JNIEnv **env, JavaVM **jvm)
+{
+    (void)java_home;  /* used only with J2ME */
+
     fptr_JNI_CreateJavaVM JNI_CreateJavaVM_fp = (fptr_JNI_CreateJavaVM)(intptr_t)dl_dlsym(jvm_lib, "JNI_CreateJavaVM");
-
     if (JNI_CreateJavaVM_fp == NULL) {
-        dl_dlclose(jvm_lib);
         BD_DEBUG(DBG_BDJ | DBG_CRIT, "Couldn't find symbol JNI_CreateJavaVM.\n");
-        return NULL;
+        return 0;
     }
 
-    BDJAVA* bdjava = calloc(1, sizeof(BDJAVA));
-    bdjava->bd = bd;
-    bdjava->index = index;
-    bdjava->path = path;
-    bdjava->h_libjvm = jvm_lib;
-    bdjava->osd_cb = osd_cb;
-    bdjava->buf = buf;
-
     JavaVMOption* option = calloc(1, sizeof(JavaVMOption) * 20);
     int n = 0;
     JavaVMInitArgs args;
-    option[n++].optionString = str_printf("-Dbluray.vfs.root=%s", path);
     option[n++].optionString = str_printf("-Ddvb.persistent.root=%s", _bdj_persistent_root());
     option[n++].optionString = str_printf("-Dbluray.bindingunit.root=%s", _bdj_buda_root());
 
@@ -431,21 +436,55 @@ BDJAVA* bdj_open(const char *path, struct bluray *bd,
     args.options = option;
     args.ignoreUnrecognized = JNI_FALSE; // don't ignore unrecognized options
 
-    JNIEnv* env = NULL;
-    int result = JNI_CreateJavaVM_fp(&bdjava->jvm, (void**) &env, &args);
-
-//    while (n) {
-//        X_FREE(option[--n].optionString);
-//    }
+    int result = JNI_CreateJavaVM_fp(jvm, (void**) env, &args);
 
+    while (--n >= 0) {
+        X_FREE(option[n].optionString);
+    }
     X_FREE(option);
 
-    if (result != JNI_OK || !env) {
-        bdj_close(bdjava);
+    if (result != JNI_OK || !*env) {
         BD_DEBUG(DBG_BDJ | DBG_CRIT, "Failed to create new Java VM.\n");
+        return 0;
+    }
+
+    return 1;
+}
+
+BDJAVA* bdj_open(const char *path, struct bluray *bd,
+                 bdj_overlay_cb osd_cb, struct bd_argb_buffer_s *buf)
+{
+    BD_DEBUG(DBG_BDJ, "bdj_open()\n");
+
+    // first load the jvm using dlopen
+    const char *java_home = NULL;
+    void* jvm_lib = _load_jvm(&java_home);
+
+    if (!jvm_lib) {
+        BD_DEBUG(DBG_BDJ | DBG_CRIT, "Wasn't able to load JVM\n");
+        return 0;
+    }
+
+    JNIEnv* env = NULL;
+    JavaVM *jvm = NULL;
+    if (!_find_jvm(jvm_lib, &env, &jvm) && !_create_jvm(jvm_lib, java_home, &env, &jvm)) {
+        dl_dlclose(jvm_lib);
         return NULL;
     }
 
+    BDJAVA* bdjava = calloc(1, sizeof(BDJAVA));
+    bdjava->bd = bd;
+    bdjava->path = path;
+    bdjava->h_libjvm = jvm_lib;
+    bdjava->osd_cb = osd_cb;
+    bdjava->buf = buf;
+    bdjava->jvm = jvm;
+
+    if (debug_mask & DBG_JNI) {
+        int version = (int)(*env)->GetVersion(env);
+        BD_DEBUG(DBG_BDJ, "Java version: %d.%d\n", version >> 16, version & 0xffff);
+    }
+
     if (!_bdj_init(bdjava, env)) {
         bdj_close(bdjava);
         return NULL;
@@ -457,82 +496,6 @@ BDJAVA* bdj_open(const char *path, struct bluray *bd,
     return bdjava;
 }
 
-int bdj_start(BDJAVA *bdjava, unsigned title)
-{
-    JNIEnv* env;
-    int attach = 0;
-    jboolean status = JNI_FALSE;
-    jclass loader_class;
-    jmethodID load_id;
-
-    if (!bdjava) {
-        return BDJ_ERROR;
-    }
-
-    BD_DEBUG(DBG_BDJ, "bdj_start(%d)\n", title);
-
-    if ((*bdjava->jvm)->GetEnv(bdjava->jvm, (void**)&env, JNI_VERSION_1_4) != JNI_OK) {
-        (*bdjava->jvm)->AttachCurrentThread(bdjava->jvm, (void**)&env, NULL);
-        attach = 1;
-    }
-
-    if (bdj_get_method(env, &loader_class, &load_id,
-                       "org/videolan/BDJLoader", "load", "(I)Z")) {
-        status = (*env)->CallStaticBooleanMethod(env, loader_class, load_id, (jint)title);
-
-        if ((*env)->ExceptionOccurred(env)) {
-            (*env)->ExceptionDescribe(env);
-            (*env)->ExceptionClear(env);
-        }
-
-        (*env)->DeleteLocalRef(env, loader_class);
-    }
-
-    if (attach) {
-        (*bdjava->jvm)->DetachCurrentThread(bdjava->jvm);
-    }
-
-    return (status == JNI_TRUE) ? BDJ_SUCCESS : BDJ_ERROR;
-}
-
-int bdj_stop(BDJAVA *bdjava)
-{
-    JNIEnv *env;
-    int attach = 0;
-    jboolean status = JNI_FALSE;
-    jclass loader_class;
-    jmethodID unload_id;
-
-    if (!bdjava) {
-        return BDJ_ERROR;
-    }
-
-    BD_DEBUG(DBG_BDJ, "bdj_stop()\n");
-
-    if ((*bdjava->jvm)->GetEnv(bdjava->jvm, (void**)&env, JNI_VERSION_1_4) != JNI_OK) {
-        (*bdjava->jvm)->AttachCurrentThread(bdjava->jvm, (void**)&env, NULL);
-        attach = 1;
-    }
-
-    if (bdj_get_method(env, &loader_class, &unload_id,
-                       "org/videolan/BDJLoader", "unload", "()Z")) {
-        status = (*env)->CallStaticBooleanMethod(env, loader_class, unload_id);
-
-        if ((*env)->ExceptionOccurred(env)) {
-            (*env)->ExceptionDescribe(env);
-            (*env)->ExceptionClear(env);
-        }
-
-        (*env)->DeleteLocalRef(env, loader_class);
-    }
-
-    if (attach) {
-        (*bdjava->jvm)->DetachCurrentThread(bdjava->jvm);
-    }
-
-    return (status == JNI_TRUE) ? BDJ_SUCCESS : BDJ_ERROR;
-}
-
 void bdj_close(BDJAVA *bdjava)
 {
     JNIEnv *env;
@@ -564,17 +527,18 @@ void bdj_close(BDJAVA *bdjava)
             (*env)->DeleteLocalRef(env, shutdown_class);
         }
 
+        bdj_unregister_native_methods(env);
+
         if (attach) {
             (*bdjava->jvm)->DetachCurrentThread(bdjava->jvm);
         }
-
-        (*bdjava->jvm)->DestroyJavaVM(bdjava->jvm);
     }
 
     if (bdjava->h_libjvm) {
         dl_dlclose(bdjava->h_libjvm);
     }
 
+    indx_free(&bdjava->index);
     X_FREE(bdjava);
 }
 
@@ -592,6 +556,12 @@ int bdj_process_event(BDJAVA *bdjava, unsigned ev, unsigned param)
         "VK_KEY",
         "MARK",
         "PSR102",
+        "PLAYLIST",
+
+        "START",
+        "STOP",
+
+        "RATE",
     };
 
     JNIEnv* env;
