@@ -20,13 +20,24 @@
 #ifndef JNIUTIL_H_
 #define JNIUTIL_H_
 
+#include "util/attributes.h"
+
 #include <jni.h>
 #include <stdarg.h>
 
 // makes an object from the specified class name and constructor signature
-jobject bdj_make_object(JNIEnv* env, const char* name, const char* sig, ...);
+BD_PRIVATE jobject bdj_make_object(JNIEnv* env, const char* name, const char* sig, ...);
 
 // makes an array for the specified class name, all elements are initialized to null
-jobjectArray bdj_make_array(JNIEnv* env, const char* name, int count);
+BD_PRIVATE jobjectArray bdj_make_array(JNIEnv* env, const char* name, int count);
+
+// get java method
+BD_PRIVATE int bdj_get_method(JNIEnv *env, jclass *cls, jmethodID *method_id,
+                              const char *class_name, const char *method_name, const char *method_sig);
+
+// register native methods for class
+BD_PRIVATE int bdj_register_methods(JNIEnv *env, const char *class_name,
+                                    const JNINativeMethod *methods, int methods_count);
+BD_PRIVATE int bdj_unregister_methods(JNIEnv *env, const char *class_name);
 
 #endif
