@@ -20,23 +20,24 @@
 package org.bluray.storage;
 
 public class StorageManager {
-    public static StorageManager getInstance()
-    {
-        throw new Error("Not implemented");
+    public static StorageManager getInstance() {
+        synchronized (StorageManager.class) {
+            if (instance == null)
+                instance = new StorageManager();
+        }
+        return instance;
     }
 
-    protected StorageManager()
-    {
-
+    protected StorageManager() {
     }
 
-    public DataAreaInfo getBindingunitDataAreaInfo()
-    {
-        throw new Error("Not implemented");
+    public DataAreaInfo getBindingunitDataAreaInfo() {
+        return new BindingunitDataAreaInfo();
     }
 
-    public DataAreaInfo getPersistentDataAreaInfo()
-    {
-        throw new Error("Not implemented");
+    public DataAreaInfo getPersistentDataAreaInfo() {
+        return new PersistentDataAreaInfo();
     }
+
+    private static StorageManager instance = null;
 }
