@@ -21,7 +21,9 @@ package org.videolan;
 
 import java.awt.Container;
 import java.awt.EventQueue;
+import java.net.URL;
 import java.util.LinkedList;
+import java.util.HashMap;
 import java.security.AccessController;
 import java.security.PrivilegedAction;
 
@@ -51,7 +53,7 @@ public class BDJXletContext implements javax.tv.xlet.XletContext, javax.microedi
                                               entry.getInitialClass(),
                                               this);
 
-        callbackQueue = new BDJActionQueue(this.threadGroup);
+        callbackQueue = new BDJActionQueue(this.threadGroup, "CallbackQueue");
     }
 
     public Object getXletProperty(String key) {
@@ -93,22 +95,79 @@ public class BDJXletContext implements javax.tv.xlet.XletContext, javax.microedi
         return container;
     }
 
+    /*
+     * Class loader
+     */
+
     public ClassLoader getClassLoader() {
         return loader;
     }
 
+    public static ClassLoader getCurrentClassLoader() {
+        BDJXletContext ctx = BDJXletContext.getCurrentContext();
+        if (ctx == null) {
+            logger.error("getCurrentClassLoader(): no context: " + Logger.dumpStack());
+            return null;
+        }
+
+        ClassLoader cldr = (ClassLoader)ctx.getClassLoader();
+        if (cldr == null) {
+            logger.error("getCurrentClassLoader(): no class loader: " + Logger.dumpStack());
+            return null;
+        }
+
+        return cldr;
+    }
+
+    public static URL getCurrentResource(String path)
+    {
+        ClassLoader cldr = (ClassLoader)BDJXletContext.getCurrentClassLoader();
+        if (cldr == null) {
+            return null;
+        }
+
+        if (path.startsWith("./") || path.startsWith(".\\")) {
+            path = path.substring(2);
+        }
+
+        URL url = cldr.getResource(path);
+        if (url == null) {
+            logger.error("getCurrentResource(): " + path + " not found: " + Logger.dumpStack());
+            return null;
+        }
+
+        return url;
+    }
+
+    /*
+     *
+     */
+
     protected AppProxy getAppProxy() {
         return AppsDatabase.getAppsDatabase().getAppProxy(appid);
     }
 
-    public boolean isDestroyed() {
-        return AppsDatabase.getAppsDatabase().getAppProxy(appid).getState() == AppProxy.DESTROYED;
+    public boolean isReleased() {
+        return released;
     }
 
+    /*
+    public boolean isDestroyed() {
+        if (released)
+            return true;
+        AppProxy proxy = AppsDatabase.getAppsDatabase().getAppProxy(appid);
+        return proxy == null || proxy.getState() == AppProxy.DESTROYED;
+    }
+    */
+
     public BDJThreadGroup getThreadGroup() {
         return threadGroup;
     }
 
+    /*
+     * Event queues
+     */
+
     protected void setEventQueue(EventQueue eq) {
         eventQueue = eq;
     }
@@ -119,11 +178,18 @@ public class BDJXletContext implements javax.tv.xlet.XletContext, javax.microedi
 
     public boolean putCallback(BDJAction cb)
     {
-        if (!isDestroyed()) {
+        synchronized (this) {
+            if (isReleased()) {
+                logger.error("callback ignored (xlet destroyed)");
+                return false;
+            }
+            if (callbackQueue == null) {
+                logger.error("callback ignored (no queue)");
+                return false;
+            }
             callbackQueue.put(cb);
             return true;
         }
-        return false;
     }
 
     protected int numEventQueueThreads() {
@@ -134,9 +200,17 @@ public class BDJXletContext implements javax.tv.xlet.XletContext, javax.microedi
                 cnt++;
             }
         }
+        if (!released) {
+            // callbackQueue
+            cnt++;
+        }
         return cnt;
     }
 
+    /*
+     * HAVI
+     */
+
     public void setSceneFactory(HSceneFactory f) {
         sceneFactory = f;
     }
@@ -145,6 +219,53 @@ public class BDJXletContext implements javax.tv.xlet.XletContext, javax.microedi
         return sceneFactory;
     }
 
+    public static Object getXletDefaultLook(String key, Class defClass) {
+        BDJXletContext ctx = BDJXletContext.getCurrentContext();
+        if (ctx == null) {
+            logger.error("getDefaultLook(): no context: " + Logger.dumpStack());
+            return null;
+        }
+        return ctx.getDefaultLook(key, defClass);
+    }
+
+    public static void setXletDefaultLook(String key, Object look) {
+        BDJXletContext ctx = BDJXletContext.getCurrentContext();
+        if (ctx == null) {
+            logger.error("setDefaultLook(): no context: " + Logger.dumpStack());
+            return;
+        }
+        ctx.setDefaultLook(key, look);
+    }
+
+    private Object getDefaultLook(String key, Class defClass) {
+        Object look = null;
+        synchronized (defaultLooks) {
+            look = defaultLooks.get(key);
+            if (look == null) {
+                try {
+                    look = defClass.newInstance();
+                    setDefaultLook(key, look);
+                } catch (Throwable t) {
+                    logger.error("Error creating default look " + defClass.getName() + " for " + key + ": " + t);
+                }
+            }
+        }
+        return look;
+    }
+
+    private void setDefaultLook(String key, Object look) {
+        synchronized (defaultLooks) {
+            defaultLooks.remove(key);
+            if (look != null) {
+                defaultLooks.put(key, look);
+            }
+        }
+    }
+
+    /*
+     * Ixc
+     */
+
     protected void addIxcThread(Thread thread) {
         synchronized (ixcThreads) {
             ixcThreads.addLast(thread);
@@ -155,26 +276,55 @@ public class BDJXletContext implements javax.tv.xlet.XletContext, javax.microedi
             ixcThreads.remove(thread);
         }
     }
-    protected void stopIxcThreads() {
-        synchronized (ixcThreads) {
-            while (!ixcThreads.isEmpty()) {
-                Thread thread = (Thread)ixcThreads.removeFirst();
-                logger.info("Stopping remote thread " + thread);
-                thread.interrupt();
-                try {
-                    thread.join(500);
-                } catch (Throwable t) {
-                }
-                if (thread.isAlive()) {
-                    PortingHelper.stopThread(thread);
-                }
-                if (thread.isAlive()) {
-                    logger.error("Error stopping remote thread " + thread);
-                }
+
+    private static boolean waitThread(Thread thread, int timeout) {
+        long startTime = System.currentTimeMillis();
+        long endTime = startTime + timeout;
+        while (thread.isAlive() && (System.currentTimeMillis() < endTime)) {
+            try {
+                Thread.sleep(10);
+            } catch (InterruptedException e) { }
+        }
+
+        boolean result = !thread.isAlive();
+        if (!result) {
+            logger.error("waitThread timeout: " + thread);
+        }
+        return result;
+    }
+
+    public static void stopThread(Thread thread, int timeout, String type) {
+        if (!waitThread(thread, timeout)) {
+            thread.interrupt();
+            if (!waitThread(thread, 200)) {
+                PortingHelper.stopThread(thread);
+                logger.error("killing " + type + " thread " + thread);
             }
         }
+        try {
+            thread.join();
+        } catch (Throwable t) { }
     }
 
+    protected void stopIxcThreads() {
+        while (true) {
+            Thread thread;
+            synchronized (ixcThreads) {
+                if (ixcThreads.isEmpty()) {
+                    break;
+                }
+                thread = (Thread)ixcThreads.removeFirst();
+            }
+            logger.info("Stopping remote thread " + thread);
+            stopThread(thread, 1000, "Ixc");
+            removeIxcThread(thread);
+        }
+    }
+
+    /*
+     * Frame-accurate animations
+     */
+
     public void addFAA(FrameAccurateAnimation faa) {
         synchronized (faaList) {
             faaList.add(faa);
@@ -198,6 +348,10 @@ public class BDJXletContext implements javax.tv.xlet.XletContext, javax.microedi
         }
     }
 
+    /*
+     *
+     */
+
     public static BDJXletContext getCurrentContext() {
         Object obj = AccessController.doPrivileged(
                 new PrivilegedAction() {
@@ -231,6 +385,7 @@ public class BDJXletContext implements javax.tv.xlet.XletContext, javax.microedi
 
         removeAllFAA();
         stopIxcThreads();
+        defaultLooks.clear();
 
         org.dvb.io.ixc.IxcRegistry.unbindAll(this);
 
@@ -239,12 +394,7 @@ public class BDJXletContext implements javax.tv.xlet.XletContext, javax.microedi
             sceneFactory = null;
         }
 
-        try {
-            callbackQueue.finalize();
-        } catch (Throwable t) {
-        } finally {
-            //callbackQueue = null;
-        }
+        callbackQueue.shutdown();
 
         EventQueue eq = eventQueue;
         eventQueue = null;
@@ -254,11 +404,17 @@ public class BDJXletContext implements javax.tv.xlet.XletContext, javax.microedi
 
         threadGroup.stopAll(1000);
 
-        threadGroup = null;
-        loader = null;
-        container = null;
+        synchronized (this) {
+            threadGroup = null;
+            loader = null;
+            container = null;
+            callbackQueue = null;
+            defaultLooks = null;
+            released = true;
+        }
     }
 
+    private boolean released = false;
     private String[] args;
     private AppID appid;
     private BDJClassLoader loader;
@@ -268,6 +424,7 @@ public class BDJXletContext implements javax.tv.xlet.XletContext, javax.microedi
     private BDJThreadGroup threadGroup = null;
     private LinkedList ixcThreads = new LinkedList();
     private LinkedList faaList = new LinkedList();
+    private HashMap defaultLooks = new HashMap();
     private BDJActionQueue callbackQueue;
     private static final Logger logger = Logger.getLogger(BDJXletContext.class.getName());
 }
