@@ -18,11 +18,14 @@
  * <http://www.gnu.org/licenses/>.
  */
 
-#include "util/macro.h"
+#include "clpi_parse.h"
+
+#include "extdata_parse.h"
+
 #include "file/file.h"
 #include "util/bits.h"
-#include "extdata_parse.h"
-#include "clpi_parse.h"
+#include "util/macro.h"
+#include "util/logging.h"
 
 #include <stdlib.h>
 #include <string.h>
@@ -583,6 +586,14 @@ static int _parse_clpi_extension(BITSTREAM *bits, int id1, int id2, void *handle
 {
     CLPI_CL *cl = (CLPI_CL*)handle;
 
+    if (id1 == 1) {
+        if (id2 == 2) {
+            // LPCM down mix coefficient
+            //_parse_lpcm_down_mix_coeff(bits, &cl->lpcm_down_mix_coeff);
+            return 0;
+        }
+    }
+
     if (id1 == 2) {
         if (id2 == 4) {
             // Extent start point
@@ -598,6 +609,7 @@ static int _parse_clpi_extension(BITSTREAM *bits, int id1, int id2, void *handle
         }
     }
 
+    BD_DEBUG(DBG_NAV | DBG_CRIT, "_parse_clpi_extension(): unhandled extension %d.%d\n", id1, id2);
     return 0;
 }
 
