@@ -1,6 +1,6 @@
 /*
  * This file is part of libbluray
- * Copyright (C) 2010-2012  Petri Hintukainen <phintuka@users.sourceforge.net>
+ * Copyright (C) 2010-2014  Petri Hintukainen <phintuka@users.sourceforge.net>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -23,9 +23,7 @@
 #include "hdmv_insn.h"
 #include "../register.h"
 
-#include "../bdnav/index_parse.h"
 #include "util/macro.h"
-#include "util/strutl.h"
 #include "util/logging.h"
 #include "util/mutex.h"
 
@@ -46,7 +44,7 @@ struct hdmv_vm_s {
     /* state */
     uint32_t       pc;            /* program counter */
     BD_REGISTERS  *regs;          /* player registers */
-    MOBJ_OBJECT   *object;        /* currently running object code */
+    const MOBJ_OBJECT *object;    /* currently running object code */
 
     HDMV_EVENT     event[5];      /* pending events to return */
 
@@ -57,15 +55,17 @@ struct hdmv_vm_s {
     MOBJ_OBJECT   *ig_object;     /* current object from IG stream */
 
     /* object currently playing playlist */
-    MOBJ_OBJECT *playing_object;
-    int          playing_pc;
+    const MOBJ_OBJECT *playing_object;
+    uint32_t     playing_pc;
 
     /* suspended object */
-    MOBJ_OBJECT *suspended_object;
-    int          suspended_pc;
+    const MOBJ_OBJECT *suspended_object;
+    uint32_t     suspended_pc;
 
-    /* disc index (used to verify CALL_TITLE/JUMP_TITLE) */
-    INDX_ROOT   *indx;
+    /* Available titles. Used to validate CALL_TITLE/JUMP_TITLE. */
+    uint8_t  have_top_menu;
+    uint8_t  have_first_play;
+    uint16_t num_titles;
 };
 
 /*
@@ -239,22 +239,27 @@ static int _queue_event(HDMV_VM *p, hdmv_event_e event, uint32_t param)
  * vm init
  */
 
-HDMV_VM *hdmv_vm_init(const char *disc_root, BD_REGISTERS *regs, INDX_ROOT *indx)
+HDMV_VM *hdmv_vm_init(const char *disc_root, BD_REGISTERS *regs,
+                      unsigned num_titles, unsigned first_play_available, unsigned top_menu_available)
 {
     HDMV_VM *p = calloc(1, sizeof(HDMV_VM));
-    char *file;
+
+    if (!p) {
+        BD_DEBUG(DBG_CRIT, "out of memory\n");
+        return NULL;
+    }
 
     /* read movie objects */
-    file = str_printf("%s/BDMV/MovieObject.bdmv", disc_root);
-    p->movie_objects = mobj_parse(file);
-    X_FREE(file);
+    p->movie_objects = mobj_parse(disc_root);
     if (!p->movie_objects) {
         X_FREE(p);
         return NULL;
     }
 
     p->regs         = regs;
-    p->indx         = indx;
+    p->num_titles      = num_titles;
+    p->have_top_menu   = top_menu_available;
+    p->have_first_play = first_play_available;
 
     bd_mutex_init(&p->mutex);
 
@@ -406,7 +411,7 @@ static int _resume_object(HDMV_VM *p, int psr_restore)
     p->suspended_object = NULL;
 
     BD_DEBUG(DBG_HDMV, "resuming object %ld at %d\n",
-             (long)((p->movie_objects->objects - p->object) / sizeof(p->movie_objects->objects[0])),
+             (long)(p->object - p->movie_objects->objects),
              p->pc);
 
     _queue_event(p, HDMV_EVENT_PLAY_STOP, 0);
@@ -419,29 +424,26 @@ static int _resume_object(HDMV_VM *p, int psr_restore)
  * branching
  */
 
-static int _is_valid_title(HDMV_VM *p, int title)
+static int _is_valid_title(HDMV_VM *p, uint32_t title)
 {
-    if (title == 0 || title == 0xffff) {
-        INDX_PLAY_ITEM *pi = (!title) ? &p->indx->top_menu : &p->indx->first_play;
-
-        if (pi->object_type == indx_object_type_hdmv &&  pi->hdmv.id_ref == 0xffff) {
-            /* no top menu or first play title (5.2.3.3) */
-            return 0;
-        }
-        return 1;
+    if (title == 0) {
+        return p->have_top_menu;
+    }
+    if (title == 0xffff) {
+        return p->have_first_play;
     }
 
-    return title > 0 && title <= p->indx->num_titles;
+    return title > 0 && title <= p->num_titles;
 }
 
-static int _jump_object(HDMV_VM *p, int object)
+static int _jump_object(HDMV_VM *p, uint32_t object)
 {
-    if (object < 0 || object >= p->movie_objects->num_objects) {
-        BD_DEBUG(DBG_HDMV|DBG_CRIT, "_jump_object(): invalid object %d\n", object);
+    if (object >= p->movie_objects->num_objects) {
+        BD_DEBUG(DBG_HDMV|DBG_CRIT, "_jump_object(): invalid object %u\n", object);
         return -1;
     }
 
-    BD_DEBUG(DBG_HDMV, "_jump_object(): jumping to object %d\n", object);
+    BD_DEBUG(DBG_HDMV, "_jump_object(): jumping to object %u\n", object);
 
     _queue_event(p, HDMV_EVENT_PLAY_STOP, 0);
 
@@ -457,10 +459,10 @@ static int _jump_object(HDMV_VM *p, int object)
     return 0;
 }
 
-static int _jump_title(HDMV_VM *p, int title)
+static int _jump_title(HDMV_VM *p, uint32_t title)
 {
     if (_is_valid_title(p, title)) {
-        BD_DEBUG(DBG_HDMV, "_jump_title(%d)\n", title);
+        BD_DEBUG(DBG_HDMV, "_jump_title(%u)\n", title);
 
         /* discard suspended object */
         p->suspended_object = NULL;
@@ -471,25 +473,29 @@ static int _jump_title(HDMV_VM *p, int title)
         return 0;
     }
 
-    BD_DEBUG(DBG_HDMV|DBG_CRIT, "_jump_title(%d): invalid title number\n", title);
+    BD_DEBUG(DBG_HDMV|DBG_CRIT, "_jump_title(%u): invalid title number\n", title);
 
     return -1;
 }
 
-static int _call_object(HDMV_VM *p, int object)
+static int _call_object(HDMV_VM *p, uint32_t object)
 {
-    BD_DEBUG(DBG_HDMV, "_call_object(%d)\n", object);
+    if (object >= p->movie_objects->num_objects) {
+        BD_DEBUG(DBG_HDMV|DBG_CRIT, "_call_object(): invalid object %u\n", object);
+        return -1;
+    }
+
+    BD_DEBUG(DBG_HDMV, "_call_object(%u)\n", object);
 
-    _queue_event(p, HDMV_EVENT_PLAY_STOP, 0);
     _suspend_object(p, 1);
 
     return _jump_object(p, object);
 }
 
-static int _call_title(HDMV_VM *p, int title)
+static int _call_title(HDMV_VM *p, uint32_t title)
 {
     if (_is_valid_title(p, title)) {
-        BD_DEBUG(DBG_HDMV, "_call_title(%d)\n", title);
+        BD_DEBUG(DBG_HDMV, "_call_title(%u)\n", title);
 
         _suspend_object(p, 1);
 
@@ -498,7 +504,7 @@ static int _call_title(HDMV_VM *p, int title)
         return 0;
     }
 
-    BD_DEBUG(DBG_HDMV|DBG_CRIT, "_call_title(%d): invalid title number\n", title);
+    BD_DEBUG(DBG_HDMV|DBG_CRIT, "_call_title(%u): invalid title number\n", title);
 
     return -1;
 }
@@ -510,14 +516,14 @@ static int _call_title(HDMV_VM *p, int title)
 static int _play_at(HDMV_VM *p, int playlist, int playitem, int playmark)
 {
     if (p->ig_object && playlist >= 0) {
-        BD_DEBUG(DBG_HDMV, "play_at(list %d, item %d, mark %d): "
+        BD_DEBUG(DBG_HDMV | DBG_CRIT, "play_at(list %d, item %d, mark %d): "
               "playlist change not allowed in interactive composition\n",
               playlist, playitem, playmark);
         return -1;
     }
 
     if (!p->ig_object && playlist < 0) {
-        BD_DEBUG(DBG_HDMV, "play_at(list %d, item %d, mark %d): "
+        BD_DEBUG(DBG_HDMV | DBG_CRIT, "play_at(list %d, item %d, mark %d): "
               "playlist not given in movie object (link commands not allowed)\n",
               playlist, playitem, playmark);
         return -1;
@@ -545,7 +551,7 @@ static int _play_at(HDMV_VM *p, int playlist, int playitem, int playmark)
 static int _play_stop(HDMV_VM *p)
 {
     if (!p->ig_object) {
-        BD_DEBUG(DBG_HDMV, "_play_stop() not allowed in movie object\n");
+        BD_DEBUG(DBG_HDMV | DBG_CRIT, "_play_stop() not allowed in movie object\n");
         return -1;
     }
 
@@ -846,7 +852,7 @@ static void _hdmv_trace_cmd(int pc, MOBJ_CMD *cmd)
 
         dst += sprintf(dst, "%04d:  ", pc);
 
-        dst += mobj_sprint_cmd(dst, cmd);
+        /*dst +=*/ mobj_sprint_cmd(dst, cmd);
 
         BD_DEBUG(DBG_HDMV, "%s\n", buf);
     }
@@ -866,7 +872,7 @@ static void _hdmv_trace_res(uint32_t new_src, uint32_t new_dst, uint32_t orig_sr
             if (new_src != orig_src) {
                 dst += sprintf(dst, " src 0x%x <== 0x%x ", orig_src, new_src);
             }
-            dst += sprintf(dst, "]");
+            /*dst +=*/ sprintf(dst, "]");
 
             BD_DEBUG(DBG_HDMV, "%s\n", buf);
         }
@@ -1077,9 +1083,14 @@ static int _hdmv_step(HDMV_VM *p)
  * interface
  */
 
-int hdmv_vm_select_object(HDMV_VM *p, int object)
+int hdmv_vm_select_object(HDMV_VM *p, uint32_t object)
 {
     int result;
+
+    if (!p) {
+        return -1;
+    }
+
     bd_mutex_lock(&p->mutex);
 
     result = _jump_object(p, object);
@@ -1088,9 +1099,39 @@ int hdmv_vm_select_object(HDMV_VM *p, int object)
     return result;
 }
 
+static int _set_object(HDMV_VM *p, int num_nav_cmds, void *nav_cmds)
+{
+    MOBJ_OBJECT *ig_object = calloc(1, sizeof(MOBJ_OBJECT));
+    if (!ig_object) {
+        BD_DEBUG(DBG_CRIT, "out of memory\n");
+        return -1;
+    }
+
+    ig_object->num_cmds = num_nav_cmds;
+    ig_object->cmds     = calloc(num_nav_cmds, sizeof(MOBJ_CMD));
+    if (!ig_object->cmds) {
+        BD_DEBUG(DBG_CRIT, "out of memory\n");
+        X_FREE(ig_object);
+        return -1;
+    }
+
+    memcpy(ig_object->cmds, nav_cmds, num_nav_cmds * sizeof(MOBJ_CMD));
+
+    p->pc        = 0;
+    p->ig_object = ig_object;
+    p->object    = ig_object;
+
+    return 0;
+}
+
 int hdmv_vm_set_object(HDMV_VM *p, int num_nav_cmds, void *nav_cmds)
 {
     int result = -1;
+
+    if (!p) {
+        return -1;
+    }
+
     bd_mutex_lock(&p->mutex);
 
     p->object = NULL;
@@ -1098,16 +1139,7 @@ int hdmv_vm_set_object(HDMV_VM *p, int num_nav_cmds, void *nav_cmds)
     _free_ig_object(p);
 
     if (nav_cmds && num_nav_cmds > 0) {
-        MOBJ_OBJECT *ig_object = calloc(1, sizeof(MOBJ_OBJECT));
-        ig_object->num_cmds = num_nav_cmds;
-        ig_object->cmds     = calloc(num_nav_cmds, sizeof(MOBJ_CMD));
-        memcpy(ig_object->cmds, nav_cmds, num_nav_cmds * sizeof(MOBJ_CMD));
-
-        p->pc        = 0;
-        p->ig_object = ig_object;
-        p->object    = ig_object;
-
-        result = 0;
+        result = _set_object(p, num_nav_cmds, nav_cmds);
     }
 
     bd_mutex_unlock(&p->mutex);
@@ -1129,6 +1161,11 @@ int hdmv_vm_get_event(HDMV_VM *p, HDMV_EVENT *ev)
 int hdmv_vm_running(HDMV_VM *p)
 {
     int result;
+
+    if (!p) {
+        return 0;
+    }
+
     bd_mutex_lock(&p->mutex);
 
     result = !!p->object;
@@ -1140,7 +1177,11 @@ int hdmv_vm_running(HDMV_VM *p)
 uint32_t hdmv_vm_get_uo_mask(HDMV_VM *p)
 {
     uint32_t     mask = 0;
-    MOBJ_OBJECT *o    = NULL;
+    const MOBJ_OBJECT *o = NULL;
+
+    if (!p) {
+        return 0;
+    }
 
     bd_mutex_lock(&p->mutex);
 
@@ -1156,6 +1197,11 @@ uint32_t hdmv_vm_get_uo_mask(HDMV_VM *p)
 int hdmv_vm_resume(HDMV_VM *p)
 {
     int result;
+
+    if (!p) {
+        return -1;
+    }
+
     bd_mutex_lock(&p->mutex);
 
     result = _resume_from_play_pl(p);
@@ -1167,6 +1213,11 @@ int hdmv_vm_resume(HDMV_VM *p)
 int hdmv_vm_suspend_pl(HDMV_VM *p)
 {
     int result = -1;
+
+    if (!p) {
+        return -1;
+    }
+
     bd_mutex_lock(&p->mutex);
 
     if (p->object || p->ig_object) {
@@ -1256,6 +1307,11 @@ static int _vm_run(HDMV_VM *p, HDMV_EVENT *ev)
 int hdmv_vm_run(HDMV_VM *p, HDMV_EVENT *ev)
 {
     int result;
+
+    if (!p) {
+        return -1;
+    }
+
     bd_mutex_lock(&p->mutex);
 
     result = _vm_run(p, ev);
