@@ -17,11 +17,13 @@
  * <http://www.gnu.org/licenses/>.
  */
 
+#include "mobj_parse.h"
+
 #include "file/file.h"
 #include "util/bits.h"
 #include "util/logging.h"
 #include "util/macro.h"
-#include "mobj_parse.h"
+#include "util/strutl.h"
 
 #include <stdlib.h>
 #include <string.h>
@@ -87,6 +89,10 @@ static int _mobj_parse_object(BITSTREAM *bs, MOBJ_OBJECT *obj)
 
     obj->num_cmds = bs_read(bs, 16);
     obj->cmds     = calloc(obj->num_cmds, sizeof(MOBJ_CMD));
+    if (!obj->cmds) {
+        BD_DEBUG(DBG_CRIT, "out of memory\n");
+        return 0;
+    }
 
     for (i = 0; i < obj->num_cmds; i++) {
         uint8_t buf[12];
@@ -101,12 +107,14 @@ void mobj_free(MOBJ_OBJECTS **p)
 {
     if (p && *p) {
 
-        int i;
-        for (i = 0 ; i < (*p)->num_objects; i++) {
-            X_FREE((*p)->objects[i].cmds);
-        }
+        if ((*p)->objects) {
+            int i;
+            for (i = 0 ; i < (*p)->num_objects; i++) {
+                X_FREE((*p)->objects[i].cmds);
+            }
 
-        X_FREE((*p)->objects);
+            X_FREE((*p)->objects);
+        }
 
         X_FREE(*p);
     }
@@ -134,6 +142,10 @@ static MOBJ_OBJECTS *_mobj_parse(const char *file_name)
         goto error;
     }
 
+    if (extension_data_start) {
+        BD_DEBUG(DBG_NAV | DBG_CRIT, "MovieObject.bdmv: unknown extension data at %d\n", extension_data_start);
+    }
+
     bs_seek_byte(&bs, 40);
 
     data_len = bs_read(&bs, 32);
@@ -143,12 +155,21 @@ static MOBJ_OBJECTS *_mobj_parse(const char *file_name)
         goto error;
     }
 
+    objects = calloc(1, sizeof(MOBJ_OBJECTS));
+    if (!objects) {
+        BD_DEBUG(DBG_CRIT, "out of memory\n");
+        goto error;
+    }
+
     bs_skip(&bs, 32); /* reserved */
     num_objects = bs_read(&bs, 16);
 
-    objects = calloc(1, sizeof(MOBJ_OBJECTS));
     objects->num_objects = num_objects;
     objects->objects = calloc(num_objects, sizeof(MOBJ_OBJECT));
+    if (!objects->objects) {
+        BD_DEBUG(DBG_CRIT, "out of memory\n");
+        goto error;
+    }
 
     for (i = 0; i < objects->num_objects; i++) {
         if (!_mobj_parse_object(&bs, &objects->objects[i])) {
@@ -167,22 +188,21 @@ static MOBJ_OBJECTS *_mobj_parse(const char *file_name)
     return NULL;
 }
 
-MOBJ_OBJECTS *mobj_parse(const char *file_name)
+MOBJ_OBJECTS *mobj_parse(const char *disc_root)
 {
-    MOBJ_OBJECTS *objects = _mobj_parse(file_name);
+    MOBJ_OBJECTS *objects;
+    char *file;
 
-    /* if failed, try backup file */
-    if (!objects) {
-        size_t len    = strlen(file_name);
-        char  *backup = malloc(len + 8);
-
-        strcpy(backup, file_name);
-        strcpy(backup + len - 16, "BACKUP/MovieObject.bdmv");
-
-        objects = _mobj_parse(backup);
-
-        X_FREE(backup);
+    file = str_printf("%s/BDMV/MovieObject.bdmv", disc_root);
+    objects = _mobj_parse(file);
+    X_FREE(file);
+    if (objects) {
+        return objects;
     }
 
+    /* if failed, try backup file */
+    file = str_printf("%s/BDMV/BACKUP/MovieObject.bdmv", disc_root);
+    objects = _mobj_parse(file);
+    X_FREE(file);
     return objects;
 }
